#include "../cvutil/cvutil.h"
#include <iostream>
#include <sstream>


#pragma comment(lib, "opencv_calib3d220.lib")
#pragma comment(lib, "opencv_imgproc220.lib")


using namespace std;

static const char *PATH = "C:/Users/AISL/Documents/sugiyama/ac022mini/";
static const char *SUFFIX = "bmp";

static const int IMAGE_NUM = 11;
static const int PAT_ROW = 7;
static const int PAT_COL = 10;
static const int PAT_SIZE = PAT_ROW * PAT_COL;
static const int ALL_POINTS = IMAGE_NUM * PAT_SIZE;
static const double CHESS_SIZE = 24.0;	// [mm]


int found_num = 0;
int corner_count = 0;
CvPoint2D32f *corners;


struct data_set
{
	float uv[PAT_SIZE][2];
	int corner_count;
};

void onMouse(int event, int x, int y, int flags, void* param)
{
	struct data_set *data = (struct data_set *) param;

	if(event == CV_EVENT_LBUTTONUP){
		data->uv[data->corner_count][0] = 0.5f*x;
		data->uv[data->corner_count][1] = 0.5f*y;
		printf("[%d] (%f, %f) \n", data->corner_count, data->uv[data->corner_count][0], data->uv[data->corner_count][1]);
		data->corner_count ++;
	}
	if(event == CV_EVENT_RBUTTONUP){
		data->corner_count --;
	}
}



int main (int argc, char *argv[]){
	int i, j, k;
	int found, found_num = 0, corner_count = 0;
	int p_count[IMAGE_NUM];
	CvPoint2D32f *corners;

	CvSize pattern_size = cvSize (PAT_ROW, PAT_COL);
	corners = (CvPoint2D32f *) cvAlloc (sizeof (CvPoint2D32f) * ALL_POINTS);

	struct data_set data;
#if 1
	cvNamedWindow("source(x2)");
	cvSetMouseCallback("source(x2)", onMouse, &data);
#endif

	// `FX{[hiLu[Vp^[j̃R[i[o
	ostringstream path;
	while(found_num != IMAGE_NUM){
		path.str("");
		path << PATH << found_num << "." << SUFFIX;
		cout << "[" << path.str().c_str() << "]" << endl;
		opencv::cvutil::Image<IPL_DEPTH_8U> src_img(path.str().c_str(), CV_LOAD_IMAGE_COLOR);
		
		found = cvFindChessboardCorners(src_img, pattern_size, &corners[found_num * PAT_SIZE], &corner_count);

		// (4)R[i[ʒuTusNZxɏCC`
		opencv::cvutil::Image<IPL_DEPTH_8U> src_gray(src_img.getWidth(), src_img.getHeight(), 1);
		cvCvtColor(src_img, src_gray, CV_BGRA2GRAY);

		cvFindCornerSubPix(src_gray, &corners[found_num * PAT_SIZE], corner_count,
			cvSize (3, 3), cvSize (-1, -1), cvTermCriteria (CV_TERMCRIT_ITER | CV_TERMCRIT_EPS, 20, 0.03));

		cvDrawChessboardCorners (src_img, pattern_size, &corners[found_num * PAT_SIZE], corner_count, found);
		p_count[found_num] = corner_count;

		src_img.show("source");

		int c = cvWaitKey();
		if(found){
			cout << "[" << found_num << "]" << endl;
			cout << PAT_SIZE << "/" << corner_count << endl;
			for(int i = 0; i < corner_count; i ++)
				cout << "(" << i << ") --- (" << corners[found_num * PAT_SIZE + i].x << ", " << corners[found_num * PAT_SIZE + i].y << ")" << endl;
			found_num++;
		}
		else{
#if 1
			data.corner_count = 0;

			opencv::cvutil::Image<IPL_DEPTH_8U> src_img_x2(src_img.getWidth()*2, src_img.getHeight()*2, 3);
			cvResize(src_img, src_img_x2);

			src_img_x2.show("source(x2)");
			cvWaitKey();

			for(int i = 0; i < PAT_SIZE; ++ i){
				corners[found_num*PAT_SIZE + i].x = data.uv[i][0];
				corners[found_num*PAT_SIZE + i].y = data.uv[i][1]; 
			}
			found = data.corner_count;
		
			corner_count = data.corner_count;
					cvFindCornerSubPix(src_gray, &corners[found_num * PAT_SIZE], corner_count,
			cvSize (3, 3), cvSize (-1, -1), cvTermCriteria (CV_TERMCRIT_ITER | CV_TERMCRIT_EPS, 20, 0.03));

			cvDrawChessboardCorners (src_img, pattern_size, &corners[found_num * PAT_SIZE], corner_count, found);
			p_count[found_num] = corner_count;

			src_img.show("source");

			cout << "[" << found_num << "]" << endl;
			cout << PAT_SIZE << "/" << corner_count << endl;
			for(int i = 0; i < corner_count; i ++)
				cout << "(" << i << ") --- (" << corners[found_num * PAT_SIZE + i].x << ", " << corners[found_num * PAT_SIZE + i].y << ")" << endl;
			found_num++;
#else		
#endif
		}
	}

	if (found_num != IMAGE_NUM)    return -1;

	CvMat *intrinsic = cvCreateMat (3, 3, CV_32FC1);
	CvMat *rotation = cvCreateMat (1, 3, CV_32FC1);
	CvMat *translation = cvCreateMat (1, 3, CV_32FC1);
	CvMat *distortion = cvCreateMat (1, 8, CV_32FC1);

	// 3ԍW̐ݒ
	CvMat object_points, image_points, point_counts;

	CvPoint3D32f objects[ALL_POINTS];
	for (i = 0; i < IMAGE_NUM; i++) {
		for (j = 0; j < PAT_COL; j++) {
			for (k = 0; k < PAT_ROW; k++) {
				objects[i * PAT_SIZE + j * PAT_ROW + k].x = float(j * CHESS_SIZE);
				objects[i * PAT_SIZE + j * PAT_ROW + k].y = float(k * CHESS_SIZE);
				objects[i * PAT_SIZE + j * PAT_ROW + k].z = float(0.0);
			}
		}
	}
	
	cvInitMatHeader (&object_points, ALL_POINTS, 3, CV_32FC1, objects);
	cvInitMatHeader (&image_points, ALL_POINTS, 1, CV_32FC2, corners);
	cvInitMatHeader (&point_counts, IMAGE_NUM, 1, CV_32SC1, p_count);
	
	// p[^Cc݌W̐
	cvCalibrateCamera2 (&object_points, &image_points, &point_counts, cvSize (640, 480), intrinsic, distortion, NULL, NULL, CV_CALIB_RATIONAL_MODEL);
	
	// Op[^̐
	CvMat sub_image_points, sub_object_points;
	int base = 0;
	cvGetRows (&image_points, &sub_image_points, base, base + PAT_SIZE);
	cvGetRows (&object_points, &sub_object_points, base, base + PAT_SIZE);
	cvFindExtrinsicCameraParams2 (&sub_object_points, &sub_image_points, intrinsic, distortion, rotation, translation); 
	
	// XMLt@Cւ̏o
	CvFileStorage *fs;
	fs = cvOpenFileStorage ("camera.xml", 0, CV_STORAGE_WRITE);
	
	cvWrite (fs, "intrinsic", intrinsic);
	cvWrite (fs, "rotation", rotation);
	cvWrite (fs, "translation", translation); 
	cvWrite (fs, "distortion", distortion);
	cvReleaseFileStorage (&fs);

	return 0;
}