#pragma once


//#define WIN32_LEAN_AND_MEAN
//#include <windows.h>

#include "cvmtvector.h"
#include "cvmtmatrix.h"

#include <iostream>
using namespace std;

namespace opencv
{
namespace cvmath
{


	/**
		2Ώ̍šŗLľvZ
		@	const Matrix *A			:	2x2Ώ̍s
		@	Vector *eigen_value	: ŗLl
	*/
	void eigienSymmetric22(const Matrix<CV_64F> &A, Vector<CV_64F> &eigen_value)
	{
		const double a = A.get(0, 0), b = A.get(0, 1), c = A.get(1, 1);
		const double bb = sqrt((a + c) * (a + c) - 4 * (a*c - b*b));

		eigen_value.set(0, (a + c + bb) * 0.5);
		eigen_value.set(1, (a + c - bb) * 0.5);
	}

	/**
		3x3Ώ̍sp̃nEXz_ϊ
		Matrix *A	:	
	*/
	void householder33(Matrix<CV_64F> &A)
	{
		const double
			a10 = A.get(1, 0), a11 = A.get(1, 1), a12 = A.get(1, 2), 
			a20 = A.get(2, 0), a21 = A.get(2, 1), a22 = A.get(2, 2);
		
		// (2)
		const double s = ((a10 < 0)? +1 : -1) * sqrt(a10 * a10 + a20 * a20);

		// (3)
		double alpha = sqrt( 2*s*(s - a10) );
		if(alpha == 0.0) return;

		// (4)
		const double u1 = (a10 - s)/ alpha;
		const double u2 = a20 / alpha;

		// (5)
		//const double q0 = alpha / 2.0;
		const double q1 = a11 * u1 + a12 * u2;
		const double q2 = a21 * u1 + a22 * u2;

		// (6)
		alpha = u1 * q1+u2 * q2;

		//const double v0 = 2.0 * q0;
		const double v1 = 2 * (q1 - alpha * u1);
		const double v2 = 2 * (q2 - alpha * u2);

		// (7)
		A.set(0, 1, s);
		A.set(1, 0, s);
		A.set(0, 2, 0.0);
		A.set(2, 0, 0.0);

		A.set(1, 1, a11 - 2 * u1 * v1);
		A.set(2, 2, a22 - 2 * u2 * v2);

		alpha = a12 - u1 * v2 - v1 * u2;
		A.set(1, 2, alpha);
		A.set(2, 1, alpha);
	}


	/*
		QRp2x2šŗLľvZ
	*/
	double getEigenValue22(const double a11, const double a12, const double a21, const double a22){
		const double b = a22 + a11;
		
		double det = b*b - 4*(a22*a11 - a12*a21);
		if(det < 0) det = 0;

		const double sqrt_det = sqrt(det);
		const double eig1 = (b + sqrt_det)/2;
		const double eig2 = (b - sqrt_det)/2;
		return (fabs(a22 - eig1) < fabs(a22 - eig2)) ? eig1 : eig2;
	}


	/**
		1QR
	*/
	void NextAQ(Matrix<CV_64F> &A, Matrix<CV_64F> &Q, int size){
		int i, j;
		double alpha, s, c;
		double aii, ai1i, aij, ai1j, qji, qji1;

		for(i = 0; i < size - 1; i ++){
			aii = A.get(i, i), ai1i = A.get(i+1, i);
			alpha = sqrt(aii * aii + ai1i * ai1i);

			s = alpha ? ai1i/alpha : alpha;
			c = alpha ? aii/alpha : alpha;
			
			for(j = i + 1; j < size; j ++){
				aij = A.get(i, j), ai1j = A.get(i+1, j);
				A.set(i,   j, aij * c + ai1j * s);
				A.set(i+1, j,- aij * s +  ai1j * c);
			}

			for(j = 0; j < size; j ++){
				qji = Q.get(j, i), qji1 = Q.get(j, i+1);
				Q.set(j, i  ,  qji * c + qji1 * s);
				Q.set(j, i+1, -qji * s + qji1 * c);
			}

			A.set(i,   i, alpha);
			A.set(i+1, i, 0.0);
		}
	}


	/**
		3x3Ώ̍spQRɂŗLlvZ
	*/
	void QR33(Matrix<CV_64F> &M){

		const double eps = 1e-10;
		int size = 3;
		Matrix<CV_64F> Q(size, size);
		Vector<CV_64F> v(size);
		int i, j, k;
		double sum;

		// s M ̎OdΊp
		householder33(M);

		while(size > 1){

			if(fabs(M.get(size-1, size-2)) < eps){
				size--;
				continue;
			}

			const double u = getEigenValue22(M.get(size-2, size-2), M.get(size-2, size-1), M.get(size-1, size-2), M.get(size-1, size-1));

			for(i = 0; i < size; i ++)
				M.set(i, i, M.get(i, i) - u);

			Q.e->rows = size;
			Q.e->cols = size;

			Q.setI();

			NextAQ(M, Q, size);

			for(i = 0; i < size; i ++){
				
				for(j = 0; j < size; j ++){
					sum = 0.0;
					for(k = i; k < size; k ++)
						sum += M.get(i, k) * Q.get(k, j);
					
					v.set(j, sum);
				}
				for(j = 0; j < size; j ++)
					M.set(i, j, v.get(j));
			}

			for(i = 0; i < size; i ++)
				M.set(i, i, M.get(i, i) + u);
		}
	}


	void calculateulateEigenVector(int idx, double l, double u0, double u1, double u2, Matrix<CV_64F> &ev)
	{
		if(l == 0.0){
			ev.set(0, idx, 0.0);
			ev.set(1, idx, 0.0);
			ev.set(2, idx, 0.0);
		}
		else{
			double norm2 = sqrt(u0*u0 + u1*u1 + u2*u2);
			if(norm2 != 0.0){
				const double norm = sqrt(norm2);
				ev.set(0, idx, u0/norm);
				ev.set(1, idx, u1/norm);
				ev.set(2, idx, u2/norm);
			}
			else{
				ev.set((idx + 1) % 3, idx, 0.0);
				ev.set((idx + 2) % 3, idx, 0.0);
				ev.set(idx, idx, 1.0);
			}
		}
	}

	/**
		3x3Ώ̍sQRɂŗLľvZƌŗLxNg͓̉IȌvZ
	*/
	void calculateEigenValueAndEigenVector33(const Matrix<CV_64F> &A, Vector<CV_64F> &eigen_value, Matrix<CV_64F> &eigen_vector){
		Matrix<CV_64F> M(3, 3);
		M.set(A);

		QR33(M);	// ŗLl߂

		const double l0 = M.get(0, 0), l1 = M.get(1, 1), l2 = M.get(2, 2);	// ŗLl

		// ŗLl̐ݒ
		eigen_value.set(0, l0);
		eigen_value.set(1, l1);
		eigen_value.set(2, l2);

		
		const double 
			a = A.get(0, 0),	b = A.get(0, 1),	c = A.get(0, 2), 
												d = A.get(1, 1),	e = A.get(1, 2), 
																					f = A.get(2, 2);

		// ŗLll0ɑ΂ŗLxNg
#if 0
		calculateEigenVector(
			0, l0, 
			(a-l1)*(a-l2) + b*b + c*c,
			b*(a-l2) + (d-l1)*b + e*c,
			c*(a-l2) + e*b + (f-l1)*c,
			eigen_vector);
#else
		Vector<CV_64F> u(3);
		if(l0 == 0.0)
			u.setZero();
		else{
			u.set(0, (a-l1)*(a-l2) + b*b + c*c);
			u.set(1, b*(a-l2) + (d-l1)*b + e*c);
			u.set(2, c*(a-l2) + e*b + (f-l1)*c);
			if(u.norm() != 0.0)
				u.normalize();
			else{
				u.set(0, 1.0);
				u.set(1, 0.0);
				u.set(2, 0.0);
			}
		}
		eigen_vector.set(0, 0, u);
#endif

		// ŗLll1ɑ΂ŗLxNg
#if 0
		calculateEigenVector(
			1, l1,
			(a-l0)*(a-l2) + b*b + c*c,
			b*(a-l2) + (d-l0)*b + e*c,
			c*(a-l2) + e*b + (f-l0)*c,
			eigen_vector);
#else
		if(l1 == 0.0){
			u.setZero();
		}
		else{
			u.set(0, (a-l0)*(a-l2) + b*b + c*c);
			u.set(1, b*(a-l2) + (d-l0)*b + e*c);
			u.set(2, c*(a-l2) + e*b + (f-l0)*c);
			if(u.norm() != 0.0)
				u.normalize();
			else{
				u.set(0, 0.0);
				u.set(1, 1.0);
				u.set(2, 0.0);
			}
		}
		eigen_vector.set(0, 1, u);
#endif

#if 0
		calculateEigenVector(
			2, l2,
			(a-l0)*(a-l1) + b*b + c*c,
			b*(a-l1) + (d-l0)*b + e*c,
			c*(a-l1) + e*b + (f-l0)*c,
			eigen_vector);
#else
		// ŗLll2ɑ΂ŗLxNg
		if(l2 == 0.0){
			u.setZero();
		}
		else{
			u.set(0, (a-l0)*(a-l1) + b*b + c*c);
			u.set(1, b*(a-l1) + (d-l0)*b + e*c);
			u.set(2, c*(a-l1) + e*b + (f-l0)*c);
			if(u.norm() != 0.0)
				u.normalize();
			else{
				u.set(0, 0.0);
				u.set(1, 0.0);
				u.set(2, 1.0);
			}
		}
		eigen_vector.set(0, 2, u);
#endif
	}


}	// namespace Math
}