#pragma once


#include "cvutil.h"
#include <vector>


/**
	xO
*/
class Labeler
{
public:
	typedef struct point_t
		{
			float x, y;

		public:
			point_t()	{ x = y = 0.0f;	}
			point_t(int x_, int y_) : x((float) x_), y((float) y_) {}
			point_t(float x_, float y_) : x(x_), y(y_) {}
			point_t(const point_t &src) : x(src.x), y(src.y) {}
		} point;

	struct label_info_t
	{
	public:
		int pix_n;		// the number of the pixel with this label
		point center_of_mass;

	public:
		label_info_t() : pix_n(0) {}
	};

	typedef label_info_t LabelInfo;
	typedef std::vector<LabelInfo> LabelInfoVec;
	typedef std::vector<point> PositionStack;

private:
	int w, h;

	PositionStack ps;
	LabelInfoVec liv;
	opencv::cvutil::Image<IPL_DEPTH_32S> labeled;

	public:
	Labeler();
	~Labeler();

public:
	void initialize(int w, int h);

	int process(const opencv::cvutil::Image<IPL_DEPTH_8U> &src_gray);

public:
	const opencv::cvutil::Image<IPL_DEPTH_32S> &getLabeledImage() const
	{	return labeled;	}


	const LabelInfoVec &getLabelInfoVector() const
	{	return liv;	}

	void print() const;

private:
	void copyImage( const opencv::cvutil::Image<IPL_DEPTH_8U> &src_gray );
	void processFirstPath();
	void processSecondPath();

};
